package com.order.controller;

import com.alibaba.fastjson.JSONObject;
import com.hfkj.common.exception.ErrorCode;
import com.hfkj.common.exception.ErrorHelp;
import com.hfkj.common.exception.SysCode;
import com.hfkj.common.security.UserCenter;
import com.hfkj.common.utils.ResponseMsgUtil;
import com.hfkj.entity.BsUserCard;
import com.hfkj.model.ResponseData;
import com.hfkj.model.UserSessionObject;
import com.hfkj.model.order.OrderModel;
import com.hfkj.service.card.BsUserCardService;
import com.hfkj.service.order.BsOrderService;
import com.hfkj.service.order.OrderPayService;
import com.hfkj.service.pay.HuiPayService;
import com.hfkj.sysenum.UserCardTypeEnum;
import com.hfkj.sysenum.order.OrderPayChannelEnum;
import com.hfkj.sysenum.order.OrderPayTypeEnum;
import com.hfkj.sysenum.order.OrderStatusEnum;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import java.util.Map;

/**
 * @className: OrderController
 * @author: HuRui
 * @date: 2024/4/30
 **/
@Controller
@RequestMapping(value="/pay")
@Api(value="支付业务")
public class OrderPayController {

    Logger log = LoggerFactory.getLogger(OrderPayController.class);
    @Resource
    private BsOrderService orderService;
    @Resource
    private OrderPayService orderPayService;
    @Resource
    private UserCenter userCenter;
    @Resource
    private BsUserCardService userCardService;

    @RequestMapping(value="/wechat",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "微信")
    public ResponseData wechat(@RequestBody JSONObject body) {
        try {
            if (body == null
                || StringUtils.isBlank(body.getString("orderNo"))
                || StringUtils.isBlank(body.getString("openId"))
            ) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            // 查询订单
            OrderModel order = orderService.getDetail(body.getString("orderNo"));
            if (order == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知的订单号");
            }
            if (!order.getOrderStatus().equals(OrderStatusEnum.status1.getCode())) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "交易订单不处于待支付");
            }
            order.setPayType(OrderPayTypeEnum.type1.getCode());
            // 请求支付渠道
            Map<Object, Object> preorder = HuiPayService.preorder(body.getString("openId"), order);
            return ResponseMsgUtil.success(preorder);

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/alipay",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "支付宝")
    public ResponseData alipay(@RequestBody JSONObject body) {
        try {
            if (body == null
                    || StringUtils.isBlank(body.getString("orderNo"))
                    || StringUtils.isBlank(body.getString("userId"))
            ) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }
            // 查询订单
            OrderModel order = orderService.getDetail(body.getString("orderNo"));
            if (order == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知的订单号");
            }
            if (order.getOrderStatus().equals(OrderStatusEnum.status2.getCode())) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "交易订单不处于待支付");
            }
            order.setPayType(OrderPayTypeEnum.type2.getCode());
            // 请求支付渠道
            Map<Object, Object> preorder = HuiPayService.preorder(body.getString("userId"), order);

            return ResponseMsgUtil.success(preorder);
        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/cardPay", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "卡支付")
    public ResponseData cardPay(@RequestBody JSONObject body) {
        try {
            // 用户
            UserSessionObject userSessionObject = userCenter.getSessionModel(UserSessionObject.class);
            if (userSessionObject == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.ACCOUNT_LOGIN_NOT, "");
            }
            if (body == null
                    || StringUtils.isBlank(body.getString("orderNo"))
                    || StringUtils.isBlank(body.getString("cardNo"))) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            // 查询用户与卡号的关系
            BsUserCard card = userCardService.getCard(userSessionObject.getUser().getId(), body.getString("cardNo"));
            if (card == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未绑定卡号");
            }
            // 查询订单
            OrderModel order = orderService.getDetail(body.getString("orderNo"));
            if (!order.getOrderStatus().equals(OrderStatusEnum.status1.getCode())) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "支付失败！交易订单不处于待支付状态");
            }
            if (UserCardTypeEnum.type1.getCode() == card.getType()) {
                return ResponseMsgUtil.success(orderPayService.hltCardPay(order, card));

            } else {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知的卡号类型");
            }

        } catch (Exception e) {
            log.error("OrderPayController -> cardPay() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

}
